package com.nickman.jms.heartbeats;

/**
 *
 * <p>Title: Heartbeat Phase 2</p>
 * <p>Description: Heartbeat Implementation Using Pub/Sub JMS.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: finetix Inc.</p>
 * @author Nicholas Whitehead
 * @version 1.0
 * This is a sample Dependent Publisher. It extends HeartbeatPublisher.
 * It will attempt to connect to the database defines in the setDependentParameters method.
 * For brevity, only two constructors have implemented, and they are pass throughs to the parent.
 * In order to implement a dependent publisher, it is necessary to override: <ol>
 * <li> void setDependentParameters(Object[] parameters)
 * <li> boolean  fireDependency()
 * </ol>
 * main is overriden as a Q&D test.
 */


import javax.jms.*;
import com.nickman.jms.heartbeats.exception.*;
import java.sql.*;


public class DependentHeartbeatPublisher extends HeartbeatPublisher {

  private String jdbcDriverClass = null;
  private String jdbcURL = null;
  private String jdbcUser = null;
  private String jdbcPassword = null;


  /**
   * Direct passthrough to HeartbeatPublisher constructor
   * @param topicConnectionFactory. A valid, configured JMS TopicConnectionFactory.
   * @param topicName The name of the hot topic eg. topic://bank/atm/alarm
   * @param period The sleep period  between checking ticks or sending heartbeat messages.
   * @param message An arbitrary string that is sent with the heartbeat. Can be null.
   * @param deliveryMode The javax.jms.DeliveryMode of the published heartbeat Messages
   * @exception HeartbeatInitializationException
   *              If the heartbeat cannot be instantiated due to an initialization error.
   *              The message will be explanatory.
   */
  public DependentHeartbeatPublisher(TopicConnectionFactory topicConnectionFactory,
                            String topicName,
                            long period,
                            String message,
                            int deliveryMode) throws HeartbeatInitializationException {
    super(topicConnectionFactory, topicName, period, message, deliveryMode);
  }
  /**
   *
   * @param topicConnectionFactory. A valid, configured JMS TopicConnectionFactory.
   * @param topicName The name of the hot topic eg. topic://bank/atm/alarm
   * @param period The sleep period  between checking ticks or sending heartbeat messages.
   * @param message An arbitrary string that is sent with the heartbeat. Can be null.
   * @throws HeartbeatInitializationException
   */
  public DependentHeartbeatPublisher( TopicConnectionFactory topicConnectionFactory,
                            String topicName,
                            long period,
                            String message ) throws HeartbeatInitializationException {
    super(topicConnectionFactory, topicName, period, message);
  }

  /**
   * Sets the parameters for the fireDependency() method.
   * @param parameters The aray members should be as follows: <ol>
   * <li>String: The name of the JDBC Database driver (eg. oracle.jdbc.OracleDriver)
   * <li>String: The JDBC URL of the database to connect to. (eg. jdbc:oracle:thin:@dlsun511:1721:dbms733)
   * <li>String: The user name to connect as. (Will not use username/password is a null is passed.
   * <li>String: The password to connect with.
   * </ol>
   * @throws InvalidDependentParameters
   */
  public void setDependentParameters(Object[] parameters) throws InvalidDependentParameters {
    if(parameters.length != 4) throw new InvalidDependentParameters("Invalid Number Of Parameters:" + parameters.length  +  ". Should be 4.");
    try {
      jdbcDriverClass = (String)parameters[0];
      jdbcURL = (String)parameters[1];
      jdbcUser = (String)parameters[2];
      jdbcPassword = (String)parameters[3];
    } catch (Exception erx) {
      throw new InvalidDependentParameters("Exception Occured Processing Parameters:" + erx);
    }
  }
  /**
   * Attempts to connect to the database with the parameters defined in setDependentParameters
   * @return Returns true if connect is successful, false if it is not.
   * Will not throw an exception, but will invoke fireHeartbeatPublisherException and pass an PublisherDependencyException if an excpetion is thrown in the method.
   */
  public boolean fireDependency() {
    java.sql.Connection conn = null;
    try {
      Class.forName(jdbcDriverClass);
      if(jdbcUser==null) conn = DriverManager.getConnection(jdbcURL);
      else conn = DriverManager.getConnection(jdbcURL, jdbcUser, jdbcPassword);
      return !conn.isClosed();
    } catch (Exception erx) {
      this.fireHeartbeatPublisherException(new PublisherDependencyException(erx));
      return false;
    } finally {
      try { conn.close(); } catch (Exception erx) {}
    }
  }
  /**
   * A static main method for quick and dirty testing.
   * Arguments: <OL>
   * <li>JNDI URL (eg. ldap://myserver:389/o=nickman.com)
   * <li>JNDI Principal (eg. cn=Directory Manager)
   * <li>JNDI Credentials (eg. secret)
   * <li>JNDI ContextFactory Name (eg. com.sun.jndi.ldap.LdapCtxFactory)
   * <li>JNDI Authentication Type (eg. simple)
   * <li>JNDI Namespace for JMS Connection Factory
   * <li>Heartbeat Topic Name
   * <li>Period (ms)
   * </OL>
   * @param args
   * Example Invocation: <b>
   * java com.nickman.jms.heartbeats.DependentHeartbeatPublisher smqp://localhost:4001 admin secret com.swiftmq.jndi.InitialContextFactoryImpl simple TopicConnectionFactory MY.WORLD 5000
   * </b>
   */

  public static void main(String args[]) {
    try {
      TopicConnectionFactory tcf = (TopicConnectionFactory)getConnectionFactory(args[0], args[1], args[2], args[3], args[4], args[5]);
      DependentHeartbeatPublisher publisher = new DependentHeartbeatPublisher(tcf, args[6], Long.parseLong(args[7]), "TEST", DeliveryMode.NON_PERSISTENT);
      publisher.setDependentParameters(new Object[]{"com.pointbase.jdbc.jdbcUniversalDriver", "jdbc:pointbase://localhost/sample", "PUBLIC", "PUBLIC"});
      TestPublisherListener tbl = new TestPublisherListener(publisher);
      publisher.registerListener(tbl);
      publisher.startHeartbeatProcess();
    } catch (Exception erx) {
      System.err.println("Error:" + erx);
      System.exit(1);
    }
  }
}
