package com.nickman.jms.heartbeats;

import com.nickman.jms.heartbeats.exception.*;
import java.util.*;
import javax.jms.*;
import javax.naming.*;
import threadpool.*;


/**
 * Title:        Heartbeat Phase 2
 * Description:  Heartbeat Implementation Using Pub/Sub JMS.
 * Copyright:    Copyright (c) 2002
 * Company:      finetix Inc.
 * @author Nicholas Whitehead
 * @version 1.0
 */

public abstract class HeartbeatBase extends Thread {

  protected TopicConnectionFactory topicConnectionFactory = null;
  protected TopicConnection topicConnection = null;
  protected TopicSession topicSession = null;
  protected Topic topic = null;
 /** A boolean that indicates if the base was initialized with a connection factory or a connection **/
  protected boolean isUsingConnFactory;
  /** The name of the topic this heartbeat base is initialized with */
  protected String topicName = null;
  /** A message that can be delivered with each heartbeat */
  protected String message = null;
  /** The sleep time for sending and checking the tick. */
  protected long period = -1;
  /** boolean that controls the continuing running of the thread. */
  protected boolean run = true;
  /** boolean indicating if a heartbeat has been received */
  private boolean tick = true;
  /** A hashtable for storing registered listeners to heartbeat events */
  protected Hashtable listeners = null;
  /** boolean indicating if messages are processing */
  protected boolean process = true;
  /** Worker thread for dispatching events **/
  ThreadPool threadPool = null;



  /** Default parameterless constructor, Do not use. */
  public HeartbeatBase() {
  }

      /**
     * Base Constructor.
     * Initializes member variables and then calls initBase()
     * @param topicConnectionFactory. A valid, configured JMS TopicConnectionFactory.
     * @param topicName The name of the hot topic eg. topic://bank/atm/alarm
     * @param period The sleep period  between checking ticks or sending heartbeat messages.
     * @param message An arbitrary string that is sent with the heartbeat. Can be null.
     * @exception HeartbeatInitializationException
     *              If the heartbeat cannot be instantiated due to an initialization error.
     *              The message will be explanatory.
     * @see com.nickman.jms.heartbeats.HeartbeatSubscriber
     * @see com.nickman.jms.heartbeats.HeartbeatPublisher
     */

  public HeartbeatBase( TopicConnectionFactory topicConnectionFactory,
                        String topicName,
                        long period,
                        String message ) throws HeartbeatInitializationException {
    this.topicConnectionFactory = topicConnectionFactory;
    this.topicName = topicName;
    this.period = period;
    this.message = message;
    threadPool = new ThreadPool();
    initBase();
  }
   /**
     * Base Constructor.
     * Initializes member variables and then calls initBase()
     * @param topicConnection. A valid, configured JMS TopicConnection.
     * @param topicName The name of the hot topic eg. topic://bank/atm/alarm
     * @param period The sleep period  between checking ticks or sending heartbeat messages.
     * @param message An arbitrary string that is sent with the heartbeat. Can be null.
     * @exception HeartbeatInitializationException
     *              If the heartbeat cannot be instantiated due to an initialization error.
     *              The message will be explanatory.
     * @see com.nickman.jms.heartbeats.HeartbeatSubscriber
     * @see com.nickman.jms.heartbeats.HeartbeatPublisher
     */
  public HeartbeatBase( TopicConnection topicConnection,
                        String topicName,
                        long period,
                        String message ) throws HeartbeatInitializationException {
    this.topicConnection = topicConnection;
    this.topicName = topicName;
    this.period = period;
    this.message = message;
    threadPool = new ThreadPool();
    initBase(topicConnection);
  }

    /**
     *
     * inits the JMS connections , topics and sessions.
     * @exception HeartbeatInitializationException
     *              If the heartbeat cannot be instantiated due to an initialization error.
     *              The message will be explanatory.
     * @see com.nickman.jms.heartbeat.HeartbeatSubscriber#initBase(TopicConnection connection)
     */
  protected void initBase() throws HeartbeatInitializationException {
    listeners = new Hashtable();
    if(topicConnectionFactory==null) throw new HeartbeatInitializationException("Invalid Connection Factory");
    try {
      topicConnection = topicConnectionFactory.createTopicConnection();
      topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
      topic = topicSession.createTopic(topicName);
      isUsingConnFactory = true;
      //init();

    } catch (Exception erx) {
      throw new HeartbeatInitializationException("Exception Initializing:" + erx.getMessage());
    } finally {
      regShutdownHook();
    }
  }



    /**
     *
     * inits the JMS connections , topics and sessions.
     * @exception HeartbeatInitializationException
     *              If the heartbeat cannot be instantiated due to an initialization error.
     *              The message will be explanatory.
     *
     */
  protected void initBase(TopicConnection connection) throws HeartbeatInitializationException {
    listeners = new Hashtable();
    try {
      topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
      topic = topicSession.createTopic(topicName);
      isUsingConnFactory = false;
      //init();
    } catch (Exception erx) {
      throw new HeartbeatInitializationException("Exception Initializing:" + erx.getMessage());
    } finally {
      regShutdownHook();
    }
  }


  /**
   * closes all the JMS related resources.
   */

  protected void destroyBase()
  {
        try{topicSession.close();}catch(Exception e){}
        try{if(isUsingConnFactory)topicConnection.stop();} catch(Exception e){}
        try{if(isUsingConnFactory)topicConnection.close();} catch(Exception e){}
        threadPool.close();

  }

    /**
     * Registers listeners that will be notified of heartbeat events.
     * @param listener An instance of an object that implements HeartbeatSubscriberEventListener or HeartbeatPublisherEventListener
     * @exception InvalidEventListener
     *              If the listener type is not the correct type of listener for the current class.
     *              The required combinations are:
     *              <TABLE BORDER="1" CELLPADDING="1">
     *              <TR ALIGN="center" VALIGN="top">
     *              <TD><B>this Type</B></TD><TD><B>Listener Type</B></TD></TR>
     *              <TR ALIGN="center" VALIGN="top">
     *              <TD>Heartbeat Consumer</TD><TD>HeartbeatSubscriberEventListener</TD></TR>
     *              <TR ALIGN="center" VALIGN="top">
     *              <TD>HeartbeatPublisher</TD><TD>HeartbeatPublisherEventListener</TD></TR>
     *              </TABLE>
     *              Any other combination will throw InvalidEventListener
     * @see com.nickman.jms.heartbeat.HeartbeatBase#unregisterListener(HeartbeatEventListener listener)
     */

  public void registerListener(IHeartbeatEventListener listener) throws InvalidEventListener {
    if(this instanceof HeartbeatSubscriber) {
      if(!(listener instanceof IHeartbeatSubscriberListener)) {
        throw new InvalidEventListener("Invalid Listener Type For " + this.getClass().getName() + ":" + listener.getClass().getName());
      }
    } else if(this instanceof HeartbeatPublisher) {
      if(!(listener instanceof IHeartbeatPublisherListener)) {
        throw new InvalidEventListener("Invalid Listener Type For " + this.getClass().getName() + ":" + listener.getClass().getName());
      }
    }
    listeners.put(new Integer(listener.hashCode()), listener);
  }

  /** Declaration of shutdownHook rotine. Must be implemented by concrete child classes */
  protected abstract void regShutdownHook();
  /** Declaration of init rotine. Must be implemented by concrete child classes to perform any required initialization */
  protected abstract void init() throws HeartbeatInitializationException;
  /** Declaration of interupt routine. Must be implemented by concrete child classes
   *  to perform the apprpriate interupt of the processing thread
  */
  protected void interruptProcess() {
    this.interrupt();
  }


    /**
     * Unregisters a registered listener.
     * @param listener A registered heartbeat event listener.
     * If the listener is regsitered, it will be removed.
     * If it is not registered, the methid will return with no error.
     * @see com.nickman.jms.heartbeats.HeartbeatBase#registerListener(HeartbeatEventListener listener)
     */

  public void unregisterListener(IHeartbeatEventListener listener) {
    listeners.remove(new Integer(listener.hashCode()));
  }

    /**
     * Starts the processing thread of the heartbeat process.
     * Once the process is stopped, it cannot be started again.
     * @see com.nickman.jms.heartbeat.HeartbeatBase#stopHeartbeatProcess()
     */

  public void startHeartbeatProcess() {
    run = true;
    this.start();
  }
    /**
     * Stops the processing thread of the heartbeat process.
     * Once the process is stopped, it cannot be started again.
     * @see com.nickman.jms.heartbeat.HeartbeatBase#startHeartbeatProcess()
     */

  public void stopHeartbeatProcess() {
    run = false;
    process=false;
    interruptProcess();
  }

    /**
     * Stops or starts the work activity of the heartbeat process, but not the processing thread.
     * For each implementation, this means the following:
     *              <TABLE BORDER="1" CELLPADDING="1">
     *              <TR ALIGN="center" VALIGN="top">
     *              <TD><B>Heartbeat Process Type</B></TD><TD><B>StopMessages Does</B></TD></TR>
     *              <TR ALIGN="center" VALIGN="top">
     *              <TD>HeartbeatSubscriber</TD><TD>Stops Checking Ticks. Will not send events to listeners..</TD></TR>
     *              <TR ALIGN="center" VALIGN="top">
     *              <TD>HeartbeatPublisher</TD><TD>Sends no heartbeat messages.</TD></TR>
     *              </TABLE>
     * @param working boolean
     */


  public void setMessagesPaused(boolean working) {
    process = working;
  }

    /**
     * Checks the message status of the heartbeat process.
     * @return true if the object is processing messages, false if it is not.
     * @see com.nickman.jms.heartbeats.HeartbeatBase#stopMessages()
     * @see com.nickman.jms.heartbeats.HeartbeatBase#startMessages()
     */

  public boolean isProcessingMessages() {
    return process;
  }
    /**
     * Checks the tick status of the heartbeat process.
     * In this version, this is only meaningful on messageConsumers.
     * @return true if the object has been ticked by a heartbeat within the last period, but not confirmed.
     */

  public synchronized boolean isTick() {
    return tick;
  }
    /**
     * Setsthe tick status of the heartbeat process.
     * @param b
     */

  public synchronized void setTick(boolean b) {
     tick = b;
  }

  public long getPeriod() {
    return period;
  }

  public static ConnectionFactory getConnectionFactory(String url, String principal, String credential, String factoryName, String authType, String jmsFactoryName) throws NamingException {
    Hashtable env = new Hashtable();
    env.put(Context.SECURITY_AUTHENTICATION, authType);
    env.put(Context.INITIAL_CONTEXT_FACTORY, factoryName);
    env.put(Context.PROVIDER_URL, url);
    env.put(Context.SECURITY_PRINCIPAL, principal);
    env.put(Context.SECURITY_CREDENTIALS, credential);
    Context ctx = new InitialContext(env);
    Object obj = ctx.lookup(jmsFactoryName);
    return (ConnectionFactory)obj;
  }

}