package com.nickman.jms.heartbeats;

import javax.jms.*;
import java.util.*;
import com.nickman.jms.heartbeats.exception.*;
import threadpool.*;



/**
 * <p>Title: Heartbeat Phase 2</p>
 * <p>Description: Heartbeat Implementation Using Pub/Sub JMS.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: finetix Inc.</p>
 * @author Nicholas Whitehead
 * @version 1.0
 */


public class HeartbeatPublisher extends HeartbeatBase implements IHeartbeatPublisher, ExceptionListener {

  /** The TopicPublisher used to send heartbeat messages. */
  protected TopicPublisher topicPublisher = null;
  /** The time-to-live limit for messages sent. */
  protected long timeToLive = 0;
  /** The delivery mode of the heartbeat messages. The default is  DeliveryMode.NON_PERSISTENT */
  protected int dMode = DeliveryMode.NON_PERSISTENT;

    /**
     * HeartbeatPublisher Constructor.
     * Initializes member variables, initializes the base JMS objects and then the topicPublisher.
     * @param topicConnectionFactory. A valid, configured JMS TopicConnectionFactory.
     * @param topicName The name of the hot topic eg. topic://bank/atm/alarm
     * @param period The sleep period  between checking ticks or sending heartbeat messages.
     * @param message An arbitrary string that is sent with the heartbeat. Can be null.
     * @param deliveryMode The javax.jms.DeliveryMode of the published heartbeat Messages
     * @exception HeartbeatInitializationException
     *              If the heartbeat cannot be instantiated due to an initialization error.
     *              The message will be explanatory.
     *
     */

  public HeartbeatPublisher( TopicConnectionFactory topicConnectionFactory,
                            String topicName,
                            long period,
                            String message,
                            int deliveryMode) throws HeartbeatInitializationException {
    super(topicConnectionFactory, topicName, period, message);
    dMode = deliveryMode;
    init();



  }
    /**
     * HeartbeatPublisher Constructor.
     * Initializes member variables, initializes the base JMS objects and then the topicPublisher.
     * The Delivery Mode is defaulted to javax.jms.DeliveryMode.NON_PERSISTENT
     * @param topicConnectionFactory. A valid, configured JMS TopicConnectionFactory.
     * @param topicName The name of the hot topic eg. topic://bank/atm/alarm
     * @param period The sleep period  between checking ticks or sending heartbeat messages.
     * @param message An arbitrary string that is sent with the heartbeat. Can be null.
     * @exception HeartbeatInitializationException
     *              If the heartbeat cannot be instantiated due to an initialization error.
     *              The message will be explanatory.
     *
     */

  public HeartbeatPublisher( TopicConnectionFactory topicConnectionFactory,
                            String topicName,
                            long period,
                            String message ) throws HeartbeatInitializationException {
    super(topicConnectionFactory, topicName, period, message);
    init();

  }

    /**
     * HeartbeatPublisher Constructor.
     * Initializes member variables, initializes the base JMS objects and then the topicPublisher.
     * The Delivery Mode is defaulted to javax.jms.DeliveryMode.NON_PERSISTENT
     * @param topicConnection. A valid, configured JMS TopicConnection
     * @param topicName The name of the hot topic eg. topic://bank/atm/alarm
     * @param period The sleep period  between checking ticks or sending heartbeat messages.
     * @param message An arbitrary string that is sent with the heartbeat. Can be null.
     * @exception HeartbeatInitializationException
     *              If the heartbeat cannot be instantiated due to an initialization error.
     *              The message will be explanatory.
     */

  public HeartbeatPublisher( TopicConnection topicConnection,
                            String topicName,
                            long period,
                            String message  ) throws HeartbeatInitializationException {
    super(topicConnection, topicName, period, message);
    init();
  }
    /**
     * HeartbeatPublisher Constructor.
     * Initializes member variables, initializes the base JMS objects and then the topicPublisher.
     * @param topicConnection. A valid, configured JMS TopicConnection
     * @param topicName The name of the hot topic eg. topic://bank/atm/alarm
     * @param period The sleep period  between checking ticks or sending heartbeat messages.
     * @param message An arbitrary string that is sent with the heartbeat. Can be null.
     * @param deliveryMode The javax.jms.DeliveryMode of the published heartbeat Messages
     * @exception HeartbeatInitializationException
     *              If the heartbeat cannot be instantiated due to an initialization error.
     *              The message will be explanatory.
     */

  public HeartbeatPublisher( TopicConnection topicConnection,
                            String topicName,
                            long period,
                            String message,
                            int deliveryMode  ) throws HeartbeatInitializationException {
    super(topicConnection, topicName, period, message);
    dMode = deliveryMode;
  }


  /**
   * Implementation of HeartbeatBase.init(). All concrete implementations shoudl provide any necesary initialization routines here.
   * @throws HeartbeatInitializationException
   */
  protected void init() throws HeartbeatInitializationException {
    try {
      topicPublisher = topicSession.createPublisher(topic);
      topicPublisher.setDeliveryMode(dMode);
      topicConnection.setExceptionListener(this);
    } catch (Exception erx) {
      throw new HeartbeatInitializationException("TopicPublisher Exception:" + erx.getMessage());
    }
  }

  /**
   * Optional Implementation fo Shutdown Hook.
   * Attempts to add a JVM shutdown hook which will clean up all JMS resources.
   * If the JVM does not support shutdown hooks, the exception should be supressed and reported on System.err
   * The following resources are closed in this shutdown hook:
   * <UL>
   *  <LI> TopicPublisher
   *  <LI> TopicSession
   *  <LI> TopicConnection
   * </UL>
   */

  protected void regShutdownHook() {
    try {
      Thread th = new Thread() {
          public void run() {
            System.out.println("Shutdown Hook Activated On Instance Of Heartbeat Supplier.");
            run = false;
            try {topicPublisher.close();} catch (Exception e) {}
            try{topicSession.close();}catch(Exception e){}
            try{if(isUsingConnFactory)topicConnection.close();} catch(Exception e){}
            try{if(isUsingConnFactory)topicConnection.stop();} catch(Exception e){}
          }
        };
      th.setDaemon(true);
      //Runtime.getRuntime().addShutdownHook(th);
    } catch (java.lang.NoSuchMethodError erx) {
      System.err.println("Unable To Register Shutdown Hook For HeartbeatPublisher");
    }
  }
  /**
   * Returns the current timetolive value for messages being generated.
   * This is the time that a message will exist until purged before being retrieved.
   * @return timetolive for current messages (milliseconds)
   * @see com.nickman.jms.heartbeats.HeartbeatPublisher#setTimeToLive(long time)
   */

  public long getTimeToLive() {
    return timeToLive;
  }
  /**
   * Sets the current timetolive value for messages being generated.
   * @param timetolive for current messages (milliseconds)
   * @see com.nickman.jms.heartbeats.HeartbeatPublisher#getTimeToLive()
   */

  public void setTimeToLive(long time) {
    timeToLive = time;
  }
  /**
   * This method invokes the notifyHeartbeatPublisherStopped() method on any registered HeartbeatPublisherEventListener objects synchronously
   * when the supplier stops sending heartbeats.
   * @see com.nickman.jms.heartbeats.HeartbeatPublisherEventListener
   */

  public void fireHeartbeatPublisherStopped() {
    fireEvent(IHeartbeatPublisher.HB_STOPPED, false);
  }
  /**
   * This method invokes the notifyHeartbeatPublisherStarted method on any registered HeartbeatPublisherEventListener objects asynchronously.
   * when the supplier starts sending heartbeats.
   * @see com.entology.jms.heartbeats.HeartbeatPublisherEventListener
   */

  public void fireHeartbeatPublisherStarted() {
    fireEvent(IHeartbeatPublisher.HB_STARTED, true);
  }

  /**
   * This method invokes the notifyHeartbeatPublisherException method on any registered HeartbeatPublisherEventListener objects ssynchronously.
   * when the supplier starts sending heartbeats.
   * @see com.entology.jms.heartbeats.HeartbeatPublisherEventListener
   */

  public void fireHeartbeatPublisherException(Exception erx) {
    Enumeration enum = listeners.keys();
    IHeartbeatPublisherListener listener = null;
    while(enum.hasMoreElements()) {
      Integer i = (Integer)enum.nextElement();
      listener = (IHeartbeatPublisherListener)listeners.get(i);
      listener.notifyHeartbeatPublisherException(erx, this);
    }
  }
  /**
   * This method invokes the notifyHeartbeatPublisherHeartbeatSent method on any registered HeartbeatPublisherEventListener objects asynchronously.
   * when the supplier starts sending heartbeats.
   * @see com.entology.jms.heartbeats.HeartbeatPublisherEventListener
   */

  public void fireHeartbeatSent() {
    fireEvent(IHeartbeatPublisher.HB_SENT, true);
  }

  /**
   * JMS ExceptionListener Required Method. Any exceptions thrown by the JMS connection will be handled here.
   *
   */
  public void onException(JMSException exception) {
    fireHeartbeatPublisherException(exception);
  }

  /** Since this is a dependent publisher, this method is defined as a NO-OP */
  public void setDependentParameters(Object[] parameters) throws InvalidDependentParameters {}

  /**
   * <p>Starts the heartbeat generating process.
   * <p>The process is as follows:
   * <OL>
   *    <LI>Begin Loop.
   *    <LI>Sleeps for the period.
   *    <LI>Send heartbeat message.
   *    <LI>End Loop.
   *    <LI>Clean Up.
   * </OL>
   */

  public void run() {
    try {
      TextMessage tmessage  = topicSession.createTextMessage();
      tmessage.setText(topicName + " HEARTBEAT");
      fireHeartbeatPublisherStarted();
      while(run) {
        Thread.sleep(period);
        topicPublisher.setTimeToLive(timeToLive);
        if(process){
          if(fireDependency()) {
            topicPublisher.publish(tmessage);
            fireHeartbeatSent();
          }
        }
      }
      fireHeartbeatPublisherStopped();
      shutdown();
    } catch (Exception erx) {
      if(run) fireHeartbeatPublisherException(erx);
      else fireHeartbeatPublisherStopped();
      shutdown();
    }
  }

  /**
   * Simple wrapper to close down all processes and resources when the run() method is complete.
   */
  public void shutdown() {
    try {topicPublisher.close();} catch (Exception e) {}
    destroyBase();
  }

  /**
   * Displays some useful name and status information about the supplier.
   * @return Output is <i>Supplier:<hot topic>:<thread state>:<message processing state></i>
   */
  public String toString() {
    return "Supplier:" + topic + ":" + (run ? "Running" : "Stopped")  + ":" + (process ? "Processing" : "Stalled");
  }


  protected void fireEvent(final int eventID, boolean asynch) {
    Runnable runnable = new Runnable() {
      public void run() {
        Enumeration enum = listeners.keys();
        while(enum.hasMoreElements()) {
          Integer i = (Integer)enum.nextElement();
          IHeartbeatPublisherListener listener = (IHeartbeatPublisherListener)listeners.get(i);
          switch (eventID) {
            case IHeartbeatPublisher.HB_STARTED :
              listener.notifyHeartbeatPublisherStarted();
              break;
            case IHeartbeatPublisher.HB_STOPPED :
              listener.notifyHeartbeatPublisherStopped();
              break;
            case IHeartbeatPublisher.HB_SENT :
              listener.notifyHeartbeatPublisherHeartbeatSent();
              break;
            default:
              fireHeartbeatPublisherException(new Exception("Invalid eventID Passed to fireEvent:" + eventID ));
              break;
          }
        }
      }
    };
    if(asynch) {
      try { threadPool.addRequest(runnable); } catch (ClosedException cex) {
        fireHeartbeatPublisherException(new Exception("Heartbeat Publisher Event Exception(" + eventID + "):" + cex));
      }
    } else {
      runnable.run();
    }
  }

  /**
   * This publisher is independent, so ths method is a NO-OP.
   * @return Always returns true since this is an independent heartbeat publisher
   */
   public boolean fireDependency() {
    return true;
   }



  /**
   * A static main method for quick and dirty testing.
   * Arguments: <OL>
   * <li>JNDI URL (eg. ldap://myserver:389/o=nickman.com)
   * <li>JNDI Principal (eg. cn=Directory Manager)
   * <li>JNDI Credentials (eg. secret)
   * <li>JNDI ContextFactory Name (eg. com.sun.jndi.ldap.LdapCtxFactory)
   * <li>JNDI Authentication Type (eg. simple)
   * <li>JNDI Namespace for JMS Connection Factory
   * <li>Heartbeat Topic Name
   * <li>Period (ms)
   * </OL>
   * Example Invocation: <b>
   * java com.nickman.jms.heartbeats.HeartbeatPublisher smqp://localhost:4001 admin secret com.swiftmq.jndi.InitialContextFactoryImpl simple TopicConnectionFactory MY.WORLD 5000
   * </b>
   * @param args
   */

  public static void main(String args[]) {
    try {
      TopicConnectionFactory tcf = (TopicConnectionFactory)getConnectionFactory(args[0], args[1], args[2], args[3], args[4], args[5]);
      HeartbeatPublisher publisher = new HeartbeatPublisher(tcf, args[6], Long.parseLong(args[7]), "TEST", DeliveryMode.NON_PERSISTENT);
      TestPublisherListener tbl = new TestPublisherListener(publisher);
      publisher.registerListener(tbl);
      publisher.startHeartbeatProcess();
    } catch (Exception erx) {
      System.err.println("Error:" + erx);
      System.exit(1);
    }
  }
}