package com.nickman.jms.heartbeats;

import javax.jms.*;
import java.util.*;
import com.nickman.jms.heartbeats.exception.*;
import threadpool.*;

/**
 * The heartbeatconsumer sleeps for <i>period</i> and then checks the <i>tick</i>.
 * If the tick has not been set, a heartbeat has not been received and the consumer
 * notifies all registered listeners (HeartbeatSubscriberEventListener).
 * If the tick has been set, it is confirmed (set false).
 * The heartbeatconsumer also registers as a listener for the <i>hot topic</i>. When a message
 * is received from the hot topic, the tick is set.
 * <p>Title: Heartbeat Phase 2</p>
 * <p>Description: Heartbeat Implementation Using Pub/Sub JMS.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: finetix Inc.</p>
 * @author Nicholas Whitehead
 * @version 1.0

 */

public class HeartbeatSubscriber extends HeartbeatBase implements IHeartbeatSubscriber, ExceptionListener, MessageListener {

  /** The TopicSubscriber used to retrieve heartbeat messages. */
  protected TopicSubscriber topicSubscriber = null;
  /** a time period between failure and resumption of a heartbeat */
  protected long resumeTime = 0;
  /** the time of failure of a heartbeat */
  protected long failTime = 0;
  /** a failed/resumed state boolean. false if not in failure, true if a failure has occured. */
  protected boolean state = true;
  /** the number of ticks during a failure */
  protected int fTickCount = 0;

    /**
     * HeartbeatSubscriber Constructor.
     * Initializes member variables, initializes the base JMS objects and then the topicSubscriber.
     * @param topicConnectionFactory. A valid, configured JMS TopicConnectionFactory.
     * @param topicName The name of the hot topic eg. topic://bank/atm/alarm
     * @param period The sleep period  between checking ticks or sending heartbeat messages.
     * @param message An arbitrary string that is sent with the heartbeat. Can be null.
     * @exception HeartbeatInitializationException
     *              If the heartbeat cannot be instantiated due to an initialization error.
     *              The message will be explanatory.
     *
     */
  public HeartbeatSubscriber( TopicConnectionFactory topicConnectionFactory,
                            String topicName,
                            long period,
                            String message  ) throws HeartbeatInitializationException {
    super(topicConnectionFactory, topicName, period, message);
    init();
  }
    /**
     * HeartbeatSubscriber Constructor.
     * Initializes member variables, initializes the base JMS objects and then the topicSubscriber.
     * @param topicConnection. A valid, configured JMS TopicConnection
     * @param topicName The name of the hot topic eg. topic://bank/atm/alarm
     * @param period The sleep period  between checking ticks or sending heartbeat messages.
     * @param message An arbitrary string that is sent with the heartbeat. Can be null.
     * @exception HeartbeatClassInitializationException
     *              If the heartbeat cannot be instantiated due to an initialization error.
     *              The message will be explanatory.
     *
     */

  public HeartbeatSubscriber( TopicConnection topicConnection,
                            String topicName,
                            long period,
                            String message  ) throws HeartbeatInitializationException {
    super(topicConnection, topicName, period, message);
    init();
  }


  /**
   * Implementation of HeartbeatBase.init(). All concrete implementations shoudl provide any necesary initialization routines here.
   * @throws HeartbeatInitializationException
   */


  protected void init() throws com.nickman.jms.heartbeats.exception.HeartbeatInitializationException {
    try {
      topicSubscriber = topicSession.createSubscriber(topic);
      this.topicConnection.setExceptionListener(this);
    } catch (Exception erx) {
      throw new HeartbeatInitializationException("TopicSubscriber Exception:" + erx.getMessage());
    }
  }
  /**
   * The heartbeats can be considered to be creating a virtual session. When the first heartbeat is received, the session starts.
   * When the heartbeat is lost, the session ends.
   * @return True if the virtual session is connected.
   */
  public boolean isConnected() {
    return !state;
  }
  /**
   * This method is called when the topic delivers a message to the consumer.
   * @param message The heartbeat message delivered.
   * @see javax.jms.MessageListener
   */
  public void onMessage(Message message) {
    try {
      if(message instanceof TextMessage) {
        if(((TextMessage)message).getText().equals(topicName + " HEARTBEAT")) {
          setTick(true);
          if(state) {  // recovering from heartbeat failure
            state = false;
            resumeTime = System.currentTimeMillis() - failTime;
            fireHeartbeatResumed();
            fTickCount=0;
          }
          fireHeartbeatReceived();
        }
      }
    } catch (Exception erx) {
      setTick(false);
      this.fireHeartbeatSubscriberException(erx);
    }
  }
  /**
   * This method invokes the notifyHeartbeatResumed method on any registered HeartbeatSubscriberEventListener objects asynchronously
   * when the consumer resumes receiving heartbeats after a failure.
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   */

  public void fireHeartbeatResumed() {
    Runnable runnable = new Runnable() {
      public void run() {
        Enumeration enum = listeners.keys();
        while(enum.hasMoreElements()) {
          Integer i = (Integer)enum.nextElement();
          IHeartbeatSubscriberListener listener = (IHeartbeatSubscriberListener)listeners.get(i);
          listener.notifyHeartbeatResumed(resumeTime, fTickCount);
        }
      }
    };
    try { threadPool.addRequest(runnable); } catch (ClosedException cex) {
      fireHeartbeatSubscriberException(new Exception("Closed Event Thread Pool Exception On fireHeartbeatResumed()"));
    }
  }
  /**
   * This method invokes the notifyHeartbeatStarted method on any registered HeartbeatSubscriberEventListener objects asynchronously
   * when the consumer resumes receiving heartbeats after a failure.
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   */

  public void fireHeartbeatSubscriberStarted() {
    fireEvent(IHeartbeatSubscriber.HB_STARTED, true);
  }

  /**
   * This method invokes the notifyHeartbeatStopped method on any registered HeartbeatSubscriberEventListener objects asynchronously
   * when the consumer resumes receiving heartbeats after a failure.
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   */

  public void fireHeartbeatSubscriberStopped() {
    fireEvent(IHeartbeatSubscriber.HB_STOPPED, false);
  }

  /**
   * This method invokes the notifyHeartbeatFailed method on any registered HeartbeatSubscriberEventListener objects asynchronously
   * when the consumer resumes receiving heartbeats after a failure.
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   */

  public void fireHeartbeatFailed() {
    Runnable runnable = new Runnable() {
      public void run() {
        Enumeration enum = listeners.keys();
        while(enum.hasMoreElements()) {
          Integer i = (Integer)enum.nextElement();
          IHeartbeatSubscriberListener listener = (IHeartbeatSubscriberListener)listeners.get(i);
          listener.notifyHeartbeatFailedEvent(fTickCount);
        }
      }
    };
    try { threadPool.addRequest(runnable); } catch (ClosedException cex) {
      fireHeartbeatSubscriberException(new Exception("Closed Event Thread Pool Exception On fireHeartbeatSubscriberStopped()"));
    }
  }

    /**
   * This method invokes the notifyHeartbeatReceived method on any registered HeartbeatSubscriberEventListener objects asynchronously
   * when the consumer resumes receiving heartbeats after a failure.
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   */

  public void fireHeartbeatReceived() {
    fireEvent(IHeartbeatSubscriber.HB_RECEIVED, true);
  }

  /**
   * This method invokes the notifyHeartbeatSubscriberException method on any registered HeartbeatSubscriberEventListener objects asynchronously
   * when the consumer resumes receiving heartbeats after a failure.
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   */

  public void fireHeartbeatSubscriberException(Exception erx) {
    Enumeration enum = listeners.keys();
    while(enum.hasMoreElements()) {
      Integer i = (Integer)enum.nextElement();
      IHeartbeatSubscriberListener listener = (IHeartbeatSubscriberListener)listeners.get(i);
      listener.notifyHeartbeatSubscriberException(erx, this);
    }
  }

  /**
   * <p>Starts the heartbeat listening process.
   * <p>The process is as follows:
   * <OL>
   *    <LI>Sets tick to false.
   *    <LI>Starts message processing from the hot topic.
   *    <LI>Begin Loop.
   *    <LI>Sleeps for the period.
   *    <LI>Check tick and calls fireHeartbeatFailed() if it is false. If it is true, it sets tick to false.
   *    <LI>End Loop.
   *    <LI>Clean Up.
   * </OL>
   */
  public void run() {
    try {
      setTick(true);  // this is true so that the first heartbeat received is not a resume. This will result in a slight delay in a heartbeat failing on startup. Needs work. NJW Feb 5 2002
      topicSubscriber.setMessageListener(this);
      topicConnection.start();
      fireHeartbeatSubscriberStarted();
      while(run) {
        this.sleep(period);
        if(run && process) {
          if(!isTick()) { // heartbeat failed
            failTime = System.currentTimeMillis();
            state = true;
            fTickCount++;
            fireHeartbeatFailed();
          } else {    // heartbeat received
            setTick(false);
            state = false;
          }
        }
      }
      fireHeartbeatSubscriberStopped();
      shutdown();
    } catch (Exception erx) {
      if(run)fireHeartbeatSubscriberException(erx);
      else fireHeartbeatSubscriberStopped();
      shutdown();
    }
  }

    /**
   * Displays some useful name and status information about the consumer.
   * @return Output is <i>Consumer:<hot topic>:<thread state>:<message processing state></i>
   */
  public String toString() {
    return "Consumer:" + topic + ":" + (run ? "Running" : "Stopped") + ":" + (process ? "Processing" : "Stalled");
  }


  /**
   * Simple wrapper to close down all processes and resources when the run() method is complete.
   */

  private void shutdown() {
        try { topicSubscriber.close(); } catch (Exception e ) {}
        try { topicSession.close(); } catch (Exception e ) {}
        try { if(isUsingConnFactory)topicConnection.stop(); } catch (Exception e) {}
        try { if(isUsingConnFactory)topicConnection.close();} catch (Exception e){}
  }

    /**
   * Attempts to add a JVM shutdown hook which will clean up all JMS resources.
   * If the JVM does not support shutdown hooks, the exception should be supressed and reported on System.err
   * The following resources are closed in this shutdown hook:
   * <UL>
   *  <LI> TopicSubscriber
   *  <LI> TopicSession
   *  <LI> TopicConnection
   * </UL>
   */

  protected void regShutdownHook() {
    try {
      Thread th = new Thread() {
          public void run() {
            System.out.println("Shutdown Hook Activated On Instance Of Heartbeat Consumer.");
            run = false;
            try {topicSubscriber.close();} catch (Exception e) {}
            try{topicSession.close();}catch(Exception e){}
            try{if(isUsingConnFactory)topicConnection.stop();} catch(Exception e){}
            try{if(isUsingConnFactory)topicConnection.close();} catch(Exception e){}
          }
        };
        th.setDaemon(true);
      //Runtime.getRuntime().addShutdownHook(th);
    } catch (java.lang.NoSuchMethodError erx) {
      System.err.println("Unable To Register Shutdown Hook For HeartbeatSubscriber");
    }
  }


  /**
   * JMS ExceptionListener Required Method. Any exceptions thrown by the JMS connection will be handled here.
   *
   */
  public void onException(JMSException exception) {
    fireHeartbeatSubscriberException(exception);
  }


  protected void fireEvent(final int eventID, boolean asynch) {
    Runnable runnable = new Runnable() {
      public void run() {
        Enumeration enum = listeners.keys();
        while(enum.hasMoreElements()) {
          Integer i = (Integer)enum.nextElement();
          IHeartbeatSubscriberListener listener = (IHeartbeatSubscriberListener)listeners.get(i);
          switch (eventID) {
            case IHeartbeatSubscriber.HB_STARTED :
              listener.notifyHeartbeatSubscriberStarted();
              break;
            case IHeartbeatSubscriber.HB_STOPPED :
              listener.notifyHeartbeatSubscriberStopped();
              break;
            case IHeartbeatSubscriber.HB_RECEIVED :
              listener.notifyHeartbeatReceived();
              break;
            default:
              fireHeartbeatSubscriberException(new Exception("Invalid eventID Passed to fireEvent:" + eventID ));
              break;
          }
        }
      }
    };
    if(asynch) {
      try { threadPool.addRequest(runnable); } catch (ClosedException cex) {
        fireHeartbeatSubscriberException(new Exception("Heartbeat Subscriber Event Exception(" + eventID + "):" + cex));
      }
    } else {
      runnable.run();
    }
  }

  /**
   * A static main method for quick and dirty testing.
   * Arguments: <OL>
   * <li>JNDI URL (eg. ldap://myserver:389/o=nickman.com)
   * <li>JNDI Principal (eg. cn=Directory Manager)
   * <li>JNDI Credentials (eg. secret)
   * <li>JNDI ContextFactory Name (eg. com.sun.jndi.ldap.LdapCtxFactory)
   * <li>JNDI Authentication Type (eg. simple)
   * <li>JNDI Namespace for JMS Connection Factory
   * <li>Heartbeat Topic Name
   * <li>Period (ms)
   * </OL>
   * Example Invocation: <b>
   * java com.nickman.jms.heartbeats.HeartbeatSubscriber smqp://localhost:4001 admin secret com.swiftmq.jndi.InitialContextFactoryImpl simple TopicConnectionFactory MY.WORLD 2000
   * </b>
   * @param args
   */

  public static void main(String args[]) {
    try {
      TopicConnectionFactory tcf = (TopicConnectionFactory)getConnectionFactory(args[0], args[1], args[2], args[3], args[4], args[5]);
      HeartbeatSubscriber subscriber = new HeartbeatSubscriber(tcf, args[6], Long.parseLong(args[7]), "TEST");
      TestSubscriberListener tbl = new TestSubscriberListener();
      subscriber.registerListener(tbl);
      subscriber.startHeartbeatProcess();
    } catch (Exception erx) {
      System.err.println("Error:" + erx);
      System.exit(1);
    }
  }

}