package com.nickman.jms.heartbeats;

/**
 * Title:        Heartbeat Phase 2
 * Description:  Heartbeat Implementation Using Pub/Sub JMS.
 * Copyright:    Copyright (c) 2001
 * Company:      finetix Inc.
 * @author Nicholas Whitehead
 * @version 1.1
 * The heartbeatsupplier sleeps for <i>period</i> and then sends a heartbeat message to
 * the hot topic.
 */

import java.util.*;
import javax.jms.*;
import com.nickman.jms.heartbeats.exception.*;

public interface IHeartbeatPublisher {

  public static final int HB_STARTED = 0;
  public static final int HB_STOPPED = 1;
  public static final int HB_SENT = 2;

  /**
   * Returns the current timetolive value for messages being generated.
   * This is the time that a message will exist until purged before being retrieved.
   * @return timetolive for current messages (milliseconds)
   * @see com.nickman.jms.heartbeats.IHeartbeatPublisher#setTimeToLive(long time)
   */
  public long getTimeToLive();

  /**
   * Sets the current timetolive value for messages being generated.
   * @param timetolive for current messages (milliseconds)
   * @see com.nickman.jms.heartbeats.IHeartbeatPublisher#getTimeToLive()
   */
  public void setTimeToLive(long time);

  /**
   * This method invokes the notifyHeartbeatPublisherStopped() method on any registered HeartbeatPublisherEventListener objects
   * when the supplier stops sending heartbeats.
   * @see com.nickman.jms.heartbeats.IHeartbeatPublisherEventListener
   * @see com.nickman.jms.heartbeats.IHeartbeatPublisherEventListener#notifyHeartbeatPublisherStopped()
   */
  public void fireHeartbeatPublisherStopped();

 /**
   * This method invokes the notifyHeartbeatPublisherStarted() method on any registered HeartbeatPublisherEventListener objects
   * when the supplier starts sending heartbeats.
   * @see com.nickman.jms.heartbeats.IHeartbeatPublisherEventListener
   * @see com.nickman.jms.heartbeats.IHeartbeatPublisherEventListener#notifyHeartbeatPublisherStarted()
   */
  public void fireHeartbeatPublisherStarted();

  /**
   * This method invokes the notifyHeartbeatPublisherException(Exception erx, HeartbeatBase heartbeat) method on any registered HeartbeatPublisherEventListener objects
   * when the supplier message sending process throws and exception.
   * @param erx The exception thrown in the message sending process.
   * @see com.nickman.jms.heartbeats.IHeartbeatPublisherEventListener
   * @see com.nickman.jms.heartbeats.IHeartbeatPublisherEventListener#notifyHeartbeatPublisherException(Exception erx, HeartbeatBase heartbeat)
   */
  public void fireHeartbeatPublisherException(Exception erx);

  /**
   * This method invokes the notifyHeartbeatPublisherHeartbeatSent() method on any registered HeartbeatPublisherEventListener objects
   * when the supplier message sending process sends a heartbeat.
   * @see com.nickman.jms.heartbeats.IHeartbeatPublisherEventListener
   * @see com.nickman.jms.heartbeats.IHeartbeatPublisherEventListener#notifyHeartbeatPublisherHeartbeatSent()
   */

  public void fireHeartbeatSent();

  /**
   * This method allows a dependency to be placed on the publisher. If the methid is implemented as a NO-OP and returns true, then the publisher is independent.
   */
  public boolean fireDependency();


  /**
   * This method allows the dependency completion parameters to be passed in to the dependent publisher.
   * If the the implementation is independent, the method should be implemented as a NO-OP.
   */
  public void setDependentParameters(Object[] parameters) throws InvalidDependentParameters;

}
