package com.nickman.jms.heartbeats;

/**
 * Title:        Heartbeat Phase 2
 * Description:  Heartbeat Implementation Using Pub/Sub JMS.
 * Copyright:    Copyright (c) 2001
 * Company:      finetix Inc.
 * @author Nicholas Whitehead
 * @version 1.1
 */

import javax.jms.*;
import com.nickman.jms.heartbeats.exception.HeartbeatInitializationException;

public interface IHeartbeatSubscriber {

  public static final int HB_STARTED = 0;
  public static final int HB_STOPPED = 1;
  public static final int HB_RECEIVED = 2;


  /**
   * Returns true if the subscriber is not in a failure state, false if it is.
   */
  public boolean isConnected();

  /**
   * This method is called when the topic delivers a message to the consumer.
   * @param message The heartbeat message delivered.
   * @see javax.jms.MessageListener
   */
  public void onMessage(Message message);

  /**
   * This method invokes the notifyHeartbeatResumed(long time) method on any registered HeartbeatSubscriberEventListener objects
   * when the consumer resumes receiving heartbeats after a failure.
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener#notifyHeartbeatResumed(long time)
   */
  public void fireHeartbeatResumed();

  /**
   * This method invokes the notifyHeartbeatSubscriberStarted() method on any registered HeartbeatSubscriberEventListener objects
   * when the consumer starts listening for heartbeats.
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener#notifyHeartbeatSubscriberStarted()
   */
  public void fireHeartbeatSubscriberStarted();

  /**
   * This method invokes the notifyHeartbeatSubscriberStopped() method on any registered HeartbeatSubscriberEventListener objects
   * when the consumer stops listening for heartbeats.
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener#notifyHeartbeatSubscriberStopped()
   */
  public void fireHeartbeatSubscriberStopped();


  /**
   * This method invokes the notifyHeartbeatFailedEvent(String message) method on any registered HeartbeatSubscriberEventListener objects
   * when the consumer skips a heartbeat. It passes the message the consumer was initialized with.
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener#notifyHeartbeatFailedEvent(String message)
   */
  public void fireHeartbeatFailed();

  /**
   * This method invokes the notifyHeartbeatReceived() method on any registered HeartbeatSubscriberEventListener objects
   * when the consumer receives a heartbeat message
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener#notifyHeartbeatReceived()
   */
  public void fireHeartbeatReceived();

  /**
   * This method invokes the notifyHeartbeatSubscriberException(Exception erx, HeartbeatBase heartbeat) method on any registered HeartbeatSubscriberEventListener objects
   * when the consumer skips a heartbeat. It passes the exception and an instance of itself.
   * @param erx The exception that was raised in the Consumer process.
   * @see com.nickman.jms.heartbeats.HeartbeatSubscriberEventListener
   *
   */
  public void fireHeartbeatSubscriberException(Exception erx);

  /**
   * Registers a subscriber listener
   * @param subListener An instance of IHeartbeatSubscriberListener
   */




}